
predict.partialRankBayes <- function(object,
                                     newdata = NULL,
                                     xpre = NULL,
                                     ranefs = NA,
                                     level = 0.95,
                                     savememory = 1000,
                                        #sims = NA,
                                     context = "observed",
                                     breaks = NULL
                                     ) {
                                        # Determine number of simulations
    bdraws <- nrow(as.matrix(object$sims.list$beta))

    # Determine number and size of batches from posterior to process
    if ((savememory>0)&&(savememory<bdraws)) {
        e <- seq(savememory,bdraws,by=savememory)
        s <- seq(1,bdraws,by=savememory)
        if (length(s)>length(e))
            e <- c(e,bdraws)
    } else {
        s <- 1
        e <- bdraws
    }
    
    # Loop over batches of draws from posterior
    simtier <- simprank <- simrank <- simpmu <- simemu <- simemupre <- NULL
    for (i in 1:length(s)) {
        current <- object
        current$sims.list$alpha <- as.matrix(current$sims.list$alpha)[s[i]:e[i],]
        current$sims.list$ystar <- as.matrix(current$sims.list$ystar)[s[i]:e[i],]
        current$sims.list$beta <- as.matrix(current$sims.list$beta)[s[i]:e[i],]
        simulates <- latentranksims(current,
                                    newdata = newdata,
                                    xpre = xpre,
                                    ranefs = ranefs,
                                    breaks = breaks
                                    )
        simrank <- rbind(simrank,simulates$simrank)
        simemu <- rbind(simemu,simulates$simemu)
        if (!is.null(xpre))
          simemupre <- rbind(simemupre,simulates$simemupre)
        if (!is.null(breaks))
          simtier <- rbind(simtier,simulates$simtier)
    }

    # Calculate summaries of posterior
    meR <- apply(simrank,2,mean)
    leR <- apply(simrank,2,quantile,probs=(1-level)/2)
    ueR <- apply(simrank,2,quantile,probs=(1+level)/2)
    seeR <- apply(simrank,2,sd)
    if (!is.null(breaks)) {
      meT <- apply(simtier,2,mean)
      leT <- apply(simtier,2,quantile,probs=(1-level)/2)
      ueT <- apply(simtier,2,quantile,probs=(1+level)/2)
      seeT <- apply(simtier,2,sd)
    }
    
    # Return results
    prediction <- list(rank=meR,lower=leR,upper=ueR,se=seeR,
                       simrank=simrank,simemu=simemu)
    if (!is.null(breaks)) {
      prediction$tier <- meT
      prediction$lower.tier <- leT
      prediction$upper.tier <- ueT
      prediction$se.tier <- seeT
      prediction$simtier <- simtier
    }
    if (!is.null(xpre))
      prediction$simemupre <- simemupre
    prediction
}
    
latentranksims <- function(object,
                           newdata = NULL,
                           xpre = NULL,
                           ranefs = NA,
                           #sims = NA,
                           context = "observed",  # alternatively, "drawn"
                           breaks = NULL
                           ) {

    bdraws <- nrow(as.matrix(object$sims.list$beta))
    
    # Optionally estimate counterfactual ranks using newdata
    if (is.null(newdata)) {
        counterfactual <- FALSE
        newdata <- object$X
        # Default for fitted values is observed random effect
        if (is.na(ranefs))
            ranefs <- "observed"
    } else {
        counterfactual <- TRUE
        # Obtain fitted latent strengths for comparison (recursive call to this function)        
        fitted <- latentranksims(object,newdata=NULL,ranefs = context )#, sims=1)

        # Mean center newdata if necessary
        if (object$meancenter) {
            newdata <- newdata - matrix(object$Xmeans,nrow=nrow(newdata),ncol=ncol(newdata),byrow=TRUE)
            if (!is.null(xpre))
              xpre <- xpre - matrix(object$Xmeans,nrow=nrow(newdata),ncol=ncol(newdata),byrow=TRUE)
        }
        # Default for counterfactuals is to sample random effects from all their posteriors
        if (is.na(ranefs))
            ranefs <- "drawn"
    }
    
    # Simulate latent strengths and ranks
    simemu <- simemupre <- NULL
      #simpmu <- NULL
    for (i in 1:nrow(newdata)) {
        xcur <- newdata[i,,drop=TRUE]
        simxb <- object$sims.list$beta %*% xcur
        if (!is.null(xpre)) {
          xprecur <- xpre[i,,drop=TRUE]
          simxpreb <- object$sims.list$beta %*% xprecur
        }

        if (is.numeric(ranefs)) {
          simxb <- simxb + ranefs
          if (!is.null(xpre))
            simxpreb <- simxpreb + ranefs
        }

        if (ranefs=="observed") {
          simxb <- simxb + object$sims.list$alpha[,i]
          if (!is.null(xpre))
            simxpreb <- simxpreb + object$sims.list$alpha[,i]
        }

        simemu <- cbind(simemu,simxb)
        if (!is.null(xpre))
          simemupre <- cbind(simemupre,simxpreb)
    }

    
    if (counterfactual) {      
        simrank <- simtier <- NULL
        for (i in 1:nrow(newdata)) {

          hypsimemu <- cbind(fitted$simemu,
                             simemu[,i])
          hypsimrank <- hypsimemu
          if (!is.null(breaks))
            hypsimtier <- hypsimrank
          for (j in 1:nrow(hypsimemu)) {
            hypsimrank[j,] <- rank(-hypsimemu[j,],ties.method="random")
            if (!is.null(breaks)) {
              hypsimtier[j,] <- getTier(hypsimrank[j,],breaks=breaks)
            }
          }

          if (!is.null(xpre)) {  # counterfactual first diff
            hypsimemupre <- cbind(fitted$simemu,
                                  simemupre[,i])
            hypsimrankpre <- hypsimemupre
            if (!is.null(breaks))
              hypsimtierpre <- hypsimrankpre
            for (j in 1:nrow(hypsimemupre)) {
              hypsimrankpre[j,] <- rank(-hypsimemupre[j,],ties.method="random")
              if (!is.null(breaks)) {
                hypsimtierpre[j,] <- getTier(hypsimrankpre[j,],breaks=breaks)
              }
            }
            simrank <- cbind(simrank,hypsimrank[,ncol(hypsimrank)] - hypsimrankpre[,ncol(hypsimrankpre)])
            if (!is.null(breaks))
              simtier <- cbind(simtier,hypsimtier[,ncol(hypsimtier)] - hypsimtierpre[,ncol(hypsimtierpre)])      
          } else {  # counterfactual expected value
            simrank <- cbind(simrank,hypsimrank[,ncol(hypsimrank)])
            if (!is.null(breaks))
              simtier <- cbind(simtier,hypsimtier[,ncol(hypsimtier)])          
          }
        }
    } else {
      simrank <- simemu
      if (!is.null(breaks))
        simtier <- simrank
      for (j in 1:nrow(simemu)) {
        simrank[j,] <- rank(-simemu[j,],ties.method="random")
        if (!is.null(breaks))
          simtier[j,] <- getTier(simrank[j,],breaks=breaks)
      }
    }   

    result <- list(simrank=simrank,simemu=simemu)
    if (!is.null(breaks))
      result$simtier <- simtier
    if (counterfactual)
      result$simemupre <- simemupre
    
    result
}


ranklikpooling <- function(object,level=0.95) {
    pooling <- rep(0,nrow(object$sims.list$alpha))
    for (i in 1:nrow(object$sims.list$alpha)) {
        pooling[i] <-
       (sd(object$sims.list$alpha[i,])/sd(object$sims.list$ystar[i,]))^2
    }
    pooling <- pmin(pooling,1)
    pooling <- 1 - pooling
    list(pooling=mean(pooling),
         se=sd(pooling),
         lower=quantile(pooling,probs=(1-level)/2),
         upper=quantile(pooling,probs=(1+level)/2)
         )
}


ranktopctile <- function(x,n) {
    x$rank <- 1 - x$rank/n
    x$upper <- 1 - x$upper/n
    x$lower <- 1 - x$lower/n
    x$se <- 1 - x$se/n
    x$predrank <- 1 - x$predrank/n
    x$predupper <- 1 - x$predupper/n
    x$predlower <- 1 - x$predlower/n
    x$predse <- 1 - x$predse/n
    x$simevs <- 1 - x$simevs/n
    x$simpvs <- 1 - x$simpvs/n
    x
}


evtable <- function(x) {
    cbind(x$rank,x$upper,x$lower)
}

extractre <- function(object) {
    remat <- object$sims.list$alpha
    meanre <- apply(remat,2,mean)
    sd <- apply(remat,2,sd)
    cbind(meanre,meanre-sd,meanre+sd)
    # return names and re's
}


plotranksrl <- function(object,
                        names,
                        n,
                        interval = "confidence", # alternative is prediciton
                        limits=NA,
                        at=NA,
                        thresholds=NULL,
                        comparerank=NULL,
                        comparelower=NULL,
                        compareupper=NULL,
                        outfile="eranks",
                        maintitle="",
                        rows=50,
                        width=7,
                        plotlist=NULL,
                        col="black",
                        pch=19,
                        lwd=0.5,
                        colcompare="blue",
                        pchcompare=23,
                        lwdcompare=0.5,
                        entryheight=0.05,
                        tiercol="gray50",
                        vertlines=NULL,
                        plotitems=NULL
                        ) {
    require(tile)
     
    pobject <- ranktopctile(object,n)

    if (is.null(plotitems))
      nobj <- length(object$rank)
      plotitems <- 1:nobj
    
    if (identical(interval,"confidence")) {
        if (length(object$lower)>nobj) {
          secondInt <- TRUE
          object$lower <- t(object$lower)
          object$upper <- t(object$upper)
          pobject$lower <- t(pobject$lower)
          pobject$upper <- t(pobject$upper)
          x <- cbind(object$rank[plotitems],object$lower[plotitems,1],object$upper[plotitems,1])
          px <- cbind(pobject$rank[plotitems],pobject$lower[plotitems,1],pobject$upper[plotitems,1])*100
          x2 <- cbind(object$rank[plotitems],object$lower[plotitems,2],object$upper[plotitems,2])
          px2 <- cbind(pobject$rank[plotitems],pobject$lower[plotitems,2],pobject$upper[plotitems,2])*100
        } else {
          secondInt <- FALSE
          x <- cbind(object$rank[plotitems],object$lower[plotitems],object$upper[plotitems])
          px <- cbind(pobject$rank[plotitems],pobject$lower[plotitems],pobject$upper[plotitems])*100
        }
        
    } else {
        
        if (length(object$predlower)>nobj) {
          secondInt <- TRUE
          object$predlower <- t(object$predlower)
          object$predupper <- t(object$predupper)
          pobject$predlower <- t(pobject$predlower)
          pobject$predupper <- t(pobject$predupper)
          x <- cbind(object$predrank[plotitems],object$predlower[plotitems,1],object$predupper[plotitems,1])
          px <- cbind(pobject$predrank[plotitems],pobject$predlower[plotitems,1],pobject$predupper[plotitems,1])*100
          x2 <- cbind(object$predrank[plotitems],object$predlower[plotitems,2],object$predupper[plotitems,2])
          px2 <- cbind(pobject$predrank[plotitems],pobject$predlower[plotitems,2],pobject$predupper[plotitems,2])*100
        } else {
          secondInt <- FALSE
          x <- cbind(object$predrank[plotitems],object$predlower[plotitems],object$predupper[plotitems])
          px <- cbind(pobject$predrank[plotitems],pobject$predlower[plotitems],pobject$predupper[plotitems])*100
        }
          
    }
    
    namesx <- as.character(names[order(-x[,1])])
    namespx <- as.character(names[order(px[,1])])

    if (!is.null(comparerank)) {
      comparerank <- as.matrix(comparerank)
      comparerank <- comparerank[plotitems,,drop=FALSE]
      comparerank <- comparerankp <- comparerankx <- comparerankpx <- comparerank
        
        for (i in 1:ncol(comparerank)) {
            comparerankp[,i] <- (1-comparerank[,i]/n)*100
            comparerankx[,i] <- comparerank[order(-x[,1]),i]
            comparerankpx[,i] <- comparerankp[order(px[,1]),i]
        }
    }
    if (!is.null(comparelower)) {
      comparelower <- as.matrix(comparelower)
      comparelower <- comparelower[plotitems,,drop=FALSE]
      comparelower <- comparelowerp <- comparelowerx <- comparelowerpx <- comparelower
        
        for (i in 1:ncol(comparelower)) {
            comparelowerp[,i] <- (1-comparelower[,i]/n)*100
            comparelowerx[,i] <- comparelower[order(-x[,1]),i]
            comparelowerpx[,i] <- comparelowerp[order(px[,1]),i]        
        }
    }
    if (!is.null(compareupper)) {
      compareupper <- as.matrix(compareupper)
      compareupper <- compareupper[plotitems,,drop=FALSE]
      compareupper <- compareupperp <- compareupperx <- compareupperpx <- compareupper
        
        for (i in 1:ncol(compareupper)) {
            compareupperp[,i] <- (1-compareupper[,i]/n)*100
            compareupperx[,i] <- compareupper[order(-x[,1]),i]
            compareupperpx[,i] <- compareupperp[order(px[,1]),i]        
        }
    }

    if (secondInt) {
      sortx2 <- x2[order(-x[,1]),]
    }
    if (secondInt) {
      sortpx2 <- px2[order(px[,1]),]
    }
    
    sortx <- x[order(-x[,1]),]
    sortpx <- px[order(px[,1]),]
   
    
    if (is.na(limits)) 
        limits <- c(0,100)
    if (is.na(at)) {
        at <- c(0,20,40,60,80,100)
    }
    lab.px <- t(round((1-at/100)*n))
    nplots <- ((nrow(x)%%rows)>0) + nrow(x)%/%rows


    if (!is.null(thresholds)) {
        thresholds <- 1-(rev(sort(thresholds))/n)
        thresholds <- thresholds*100
    }
    
    for (i in 1:nplots) {
        s <- rows*(i-1) + 1
        e <- min(nrow(x), rows*i)
        outfile0 <- paste(outfile,"_",i,sep="")
        if (nplots>1) {
            maint <- paste(maintitle,", ",i,"/",nplots,sep="")
        } else {
            maint <- maintitle
        }

        xplot <- NULL
        lowerplot <- NULL
        upperplot <- NULL

        if (!is.null(comparerank)) {
            xplot <- cbind(xplot,comparerankpx[s:e,])
            if (!is.null(comparelower)) {
                lowerplot <- cbind(lowerplot,comparelowerpx[s:e,])
            } else {
                lowerplot <- cbind(lowerplot,matrix(NA,nrow=(e-s+1),ncol=ncol(comparerank)))
            }
            if (!is.null(compareupper)) {
                upperplot <- cbind(upperplot,compareupperpx[s:e,])
            } else {
                upperplot <- cbind(upperplot,matrix(NA,nrow=(e-s+1),ncol=ncol(comparerank)))
            }
            lab.px <- rbind(lab.px,lab.px)
            if (is.null(plotlist))
                plotlist <- rep(1,ncol(xplot))           
        } else {
            if (is.null(plotlist))
                plotlist <- 1
        }

        if (length(col)<length(plotlist)) {
            col <- rep(col,length.out=length(plotlist))
        }
        if (length(pch)<length(plotlist)) {
            pch <- rep(pch,length.out=length(plotlist))
        }
         if (length(lwd)<length(plotlist)) {
            lwd <- rep(lwd,length.out=length(plotlist))
        }


        # Make ropeladder traces

        mainresult <- ropeladder(x=sortpx[s:e,1],
                                 lower=sortpx[s:e,2],
                                 upper=sortpx[s:e,3],
                                 labels=namespx[s:e],
                                 pch=pch,              
                                 lwd=lwd,
                                 group=1,
                                 plot=1,
                                 col="black",
                                 entryheight = entryheight,
                                 shadebox=NA
                                 )

        if (secondInt) {
          mainresult$lwd <- mainresult$lwd + 1
          mainresult2 <- ropeladder(x=sortpx2[s:e,1],
                                 lower=sortpx2[s:e,2],
                                 upper=sortpx2[s:e,3],
                                 labels=namespx[s:e],
                                 pch=pch,              
                                 lwd=lwd,
                                 group=1,
                                 plot=1,
                                 col="black",
                                 entryheight = entryheight,
                                 shadebox=NA
                                 )
        }

       
        if (!is.null(comparerank)) {
          if (secondInt) {
            collectedtraces <- vector("list",(ncol(comparerank)+2))
            collectedtraces[[1]] <- mainresult
            collectedtraces[[2]] <- mainresult2
          } else {
            collectedtraces <- vector("list",(ncol(comparerank)+1))
            collectedtraces[[1]] <- mainresult
          }
          lowerplot <- as.matrix(lowerplot)
          upperplot <- as.matrix(upperplot)
          for (ict in 1:ncol(comparerank)) {
            nexttrace <- paste("comparisontrace",ict,sep="")
            collectedtraces[[nexttrace]] <-  ropeladder(x=xplot[,ict],
                                                        lower=lowerplot[,ict],
                                                        upper=upperplot[,ict],
                                                        labels=namespx[s:e],
                                                        pch=pchcompare,                 
                                                        lwd=lwdcompare,
                                                        col=colcompare,
                                                        group=1,
                                                        plot=1,
                                                        entryheight = entryheight,
                                                        shadebox=NA
                                                        )
          }
        } else {
          collectedtraces <- mainresult

        }

        print(collectedtraces)

        if (!is.null(thresholds)) {
          collectedpolygons <- vector("list",(length(thresholds)-1))         
          at.first <- 0
          at.last <- 100
          for (icp in 1:(length(thresholds)-1)) {
            nextpoly <- paste("polygon",icp,sep="")
            collectedpolygons[[nextpoly]] <- polygonTile(x=c(max(thresholds[icp],at.first),
                                                          min(thresholds[icp+1],at.last),
                                                          min(thresholds[icp+1],at.last),
                                                          max(thresholds[icp],at.first)),
                                                         y=c(0,0,1,1),
                                                         fill=tiercol[icp],
                                                         col=NA,
                                                         alpha=1,
                                                         plot=1,
                                                         layer = 50
                                                         )
          }
        }

        
        if (!is.null(vertlines)) {
          collectedvertlines <- vector("list",length(vertlines))         
          for (icl in 1:length(vertlines)) {
            nextlines <- paste("lines",icp,sep="")
            collectedvertlines[[nextlines]] <- linesTile(x=rep(vertlines[icl],2),
                                                     y=c(0,1),
                                                     col="black",
                                                     lty="dashed",
                                                     plot=1
                                                     )
            
          }
        }
        
        
        tc <- tile(mainresult,
                   collectedtraces,
                   collectedpolygons,
                   collectedvertlines,
                   output = list(width=width,outfile=outfile0,type="pdf"),
                   limits=limits,  
                   topaxis=list(at=at,add=TRUE),
                   topaxistitle=list(type="first",labels=c("Expected percentile")),
                   xaxis=list(at=at,labels=lab.px),
                   xaxistitle=list(type="first",labels=c("Expected rank")),
                   maintitle=list(type="first",labels=maintitle)   
                   )
}
invisible(tc)

}

plotre <- function(result,names,outfile="re",maintitle="",rows=50,width=7,pch=19,lwd=0.5,col="black",entryheight=0.05) {

  require(tile)
# Plot random effects by person

  remat <- result$sims.list$alpha
  meanre <- apply(remat,2,mean)
  sd <- apply(remat,2,sd)
  re <- cbind(meanre,meanre-sd,meanre+sd)
  namesre <- as.character(names[order(re[,1])])
  sortre <- re[order(re[,1]),]
  limits <- c(floor(min(re)),ceiling(max(re)))
  at <- seq(limits[1],limits[2],1)
  
  nplots <- ((nrow(re)%%rows)>0) + nrow(re)%/%rows
  
  dataout <- cbind(namesre,sortre)
  varsout <- c("case","re","lowerre","upperre")
  dataout <- as.data.frame(dataout)
  colnames(dataout) <- varsout
  
  write.csv(dataout,
            file=paste(outfile,".csv",sep="")
            )

  for (i in 1:nplots) {
    s <- rows*(i-1) + 1
    e <- min(nrow(re), rows*i)
    outfile0 <- paste(outfile,"_",i,sep="")
    if (nplots>1) {
      maint <- paste(maintitle,", ",i,"/",nplots,sep="")
    } else {
      maint <- maintitle
    }
    
    trace <- ropeladder(x=sortre[s:e,1],                #  The point estimates (vector)
                        lower=sortre[s:e,2],           #  The lower bounds (vector)
                        upper=sortre[s:e,3],            #  The upper bounds (vector)
                        labels=namesre[s:e],            #  Vector of labels for "steps" of ladder;
                                        #  set to NULL for no labels (default)
                        pch=pch,
                        lwd=lwd,
                        col=col,
                        group=1,
                        plot=1,
                        entryheight = entryheight,
                        shadebox=NA
                        )
                
    tile(trace,
         maintitle=list(fontface="bold",labels=c(maint)),
                                     #  Overtitles (appear above plot matrix)
         topaxis=list(at=at,add=TRUE),
         topaxistitle=list(type="first",labels=c("Random Effects")),
         xaxis=list(at=at),
         xaxistitle=list(type="first",labels=c("weaker <-      -> stronger")),        
         limits=limits,           # Plot region boundaries
         output = list(wide=width,outfile=outfile0,type="pdf"),
                                     # Output file information
         gridlines=list(type="x"),          # Gridlines controls
                #entryheight = 0.33,
         shadebox=NA,              #  Add gray shading of point estimate range?
         RxC=c(1,1)                #  Number of Rows and Columns of plots to make
         )

  }
  invisible(NULL)
}
                     
